@@ -187,6 +187,7 @@ typedef unsigned long u32;
 #define EINPROGRESS WSAEINPROGRESS
 #define EWOULDBLOCK WSAEWOULDBLOCK
 #define EISCONN WSAEISCONN
+#define EADDRINUSE WSAEADDRINUSE
 
 #ifdef EINVAL
 #undef EINVAL
@@ -322,6 +323,19 @@ extern char *sys_errlist[];
     ((num)>=sys_nerr ? "Unknown error" : sys_errlist[num]))
 #endif /* SunOS 4 */
 
+/* AIX does not have SOL_TCP defined */
+#ifndef SOL_TCP
+#define SOL_TCP SOL_SOCKET
+#endif /* SOL_TCP */
+
+/* Linux */
+#ifdef __linux__
+#ifndef IP_TRANSPARENT
+/* old kernel headers without IP_TRANSPARENT definition */
+#define IP_TRANSPARENT 19
+#endif /* IP_TRANSPARENT */
+#endif /* __linux__ */
+
 #endif /* USE_WIN32 */
 
 /**************************************** OpenSSL headers */
