@@ -1,55 +1,68 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (c) 1998-2007 Michal Trojnara <Michal.Trojnara@mirt.net>
- *                 All Rights Reserved
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
- *   (at your option) any later version.
+ *   Copyright (C) 1998-2009 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the
+ *   Free Software Foundation; either version 2 of the License, or (at your
+ *   option) any later version.
+ * 
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
- *
- *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *   In addition, as a special exception, Michal Trojnara gives
- *   permission to link the code of this program with the OpenSSL
- *   library (or with modified versions of OpenSSL that use the same
- *   license as OpenSSL), and distribute linked combinations including
- *   the two.  You must obey the GNU General Public License in all
- *   respects for all of the code used other than OpenSSL.  If you modify
- *   this file, you may extend this exception to your version of the
- *   file, but you are not obligated to do so.  If you do not wish to
- *   do so, delete this exception statement from your version.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *   See the GNU General Public License for more details.
+ * 
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, see <http://www.gnu.org/licenses>.
+ * 
+ *   Linking stunnel statically or dynamically with other modules is making
+ *   a combined work based on stunnel. Thus, the terms and conditions of
+ *   the GNU General Public License cover the whole combination.
+ * 
+ *   In addition, as a special exception, the copyright holder of stunnel
+ *   gives you permission to combine stunnel with free software programs or
+ *   libraries that are released under the GNU LGPL and with code included
+ *   in the standard release of OpenSSL under the OpenSSL License (or
+ *   modified versions of such code, with unchanged license). You may copy
+ *   and distribute such a system following the terms of the GNU GPL for
+ *   stunnel and the licenses of the other code concerned.
+ * 
+ *   Note that people who make modified versions of stunnel are not obligated
+ *   to grant this special exception for their modified versions; it is their
+ *   choice whether to do so. The GNU General Public License gives permission
+ *   to release a modified version without this exception; this exception
+ *   also makes it possible to release a modified version which carries
+ *   forward this exception.
  */
 
 #include "common.h"
 #include "prototypes.h"
 
-static DISK_FILE *outfile=NULL; /* Logging to file disabled by default */
+static void log_raw(const int, const char *, const char *);
+static void get_timestamp(const int, char *);
+
+static DISK_FILE *outfile=NULL;
+static struct LIST {
+    struct LIST *next;
+    int level;
+    char stamp[STRLEN], text[STRLEN];
+} *head=NULL, *tail=NULL;
+static enum {INIT_NONE, INIT_ERROR, INIT_FULL} mode=INIT_NONE;
 
 void log_open(void) {
-    if(options.output_file) { /* 'output' option specified */
+    if(options.output_file) /* 'output' option specified */
         outfile=file_open(options.output_file, 1);
-        if(outfile) /* file opened or created */
-            return; /* no need to setup syslog */
-    }
 #if !defined(USE_WIN32) && !defined (__vms)
-    if(options.option.syslog) {
+    if(options.option.syslog)
 #ifdef __ultrix__
         openlog("stunnel", 0);
 #else
         openlog("stunnel", LOG_CONS | LOG_NDELAY, options.facility);
 #endif /* __ultrix__ */
-    }
 #endif /* !defined (USE_WIN32) && !defined (__vms) */
-    if(options.output_file)
+    if(options.output_file && !outfile)
         s_log(LOG_ERR, "Unable to open output file: %s", options.output_file);
+    mode=INIT_FULL;
 }
 
 void log_close(void) {
@@ -63,17 +76,25 @@ void log_close(void) {
 #endif
 }
 
+void log_flush(void) {
+    struct LIST *tmp;
+
+    if(mode==INIT_NONE)
+        mode=INIT_ERROR;
+    while(head) {
+        log_raw(head->level, head->stamp, head->text);
+        tmp=head;
+        head=head->next;
+        free(tmp);
+    }
+    head=tail=NULL;
+}
+
 void s_log(int level, const char *format, ...) {
     va_list arglist;
-    char text[STRLEN], timestamped[STRLEN];
-    time_t gmt;
-    struct tm *timeptr;
-#if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
-    struct tm timestruct;
-#endif
+    char text[STRLEN], stamp[STRLEN];
+    struct LIST *tmp;
 
-    if(level>options.debug_level)
-        return;
     va_start(arglist, format);
 #ifdef HAVE_VSNPRINTF
     vsnprintf(text, STRLEN, format, arglist);
@@ -81,13 +102,67 @@ void s_log(int level, const char *format, ...) {
     vsprintf(text, format, arglist);
 #endif
     va_end(arglist);
-#if !defined (USE_WIN32) && !defined (__vms)
-    if(!outfile && options.option.syslog) {
-        syslog(level, "LOG%d[%lu:%lu]: %s",
-            level, stunnel_process_id(), stunnel_thread_id(), text);
+
+    if(mode!=INIT_NONE) { /* ready log the text directly */
+        get_timestamp(level, stamp);
+        log_raw(level, stamp, text);
         return;
     }
+
+    /* not initialized -> save the text to log it later */
+    tmp=malloc(sizeof(struct LIST));
+    if(!tmp) /* out of memory */
+        return;
+    tmp->next=NULL;
+    tmp->level=level;
+    get_timestamp(level, tmp->stamp);
+    safecopy(tmp->text, text);
+    if(tail)
+        tail->next=tmp;
+    else
+        head=tmp;
+    tail=tmp;
+}
+
+static void log_raw(const int level,
+        const char *stamp, const char *text) {
+    char stamped[STRLEN];
+
+    safecopy(stamped, stamp);
+    safeconcat(stamped, text);
+
+    if(mode==INIT_FULL && level<=options.debug_level) {
+            /* logging allowed by the configuration file */
+#if !defined (USE_WIN32) && !defined (__vms)
+        if(options.option.syslog && level!=LOG_RAW)
+            syslog(level, "LOG%d[%lu:%lu]: %s", level,
+                stunnel_process_id(), stunnel_thread_id(), text);
 #endif /* USE_WIN32, __vms */
+        if(outfile)
+            file_putline(outfile, stamped); /* send log to file */
+    }
+
+#ifdef USE_WIN32
+    if(mode==INIT_ERROR || level==LOG_RAW || level<=options.debug_level)
+        win_log(stamped); /* always log to the GUI window */
+#else /* Unix */
+    if(mode==INIT_ERROR || level==LOG_RAW ||
+            (level<=options.debug_level && options.option.foreground))
+        fprintf(stderr, "%s\n", stamped); /* send log to stderr */
+#endif
+}
+
+static void get_timestamp(const int level, char *txt) {
+    time_t gmt;
+    struct tm *timeptr;
+#if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
+    struct tm timestruct;
+#endif
+
+    if(level==LOG_RAW) { /* do not add timestamp for raw logs */
+        *txt='\0';
+        return;
+    }
     time(&gmt);
 #if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
     timeptr=localtime_r(&gmt, &timestruct);
@@ -95,37 +170,14 @@ void s_log(int level, const char *format, ...) {
     timeptr=localtime(&gmt);
 #endif
 #ifdef HAVE_SNPRINTF
-    snprintf(timestamped, STRLEN,
+    snprintf(txt, STRLEN,
 #else
-    sprintf(timestamped,
+    sprintf(txt,
 #endif
-        "%04d.%02d.%02d %02d:%02d:%02d LOG%d[%lu:%lu]: %s",
+        "%04d.%02d.%02d %02d:%02d:%02d LOG%d[%lu:%lu]: ",
         timeptr->tm_year+1900, timeptr->tm_mon+1, timeptr->tm_mday,
         timeptr->tm_hour, timeptr->tm_min, timeptr->tm_sec,
-        level, stunnel_process_id(), stunnel_thread_id(), text);
-#ifdef USE_WIN32
-    win_log(timestamped); /* always log to the GUI window */
-    if(outfile) /* fallback to stderr is not available on WIN32 */
-#endif
-        file_putline(outfile, timestamped);
-}
-
-void log_raw(const char *format, ...) {
-    va_list arglist;
-    char text[STRLEN];
-
-    va_start(arglist, format);
-#ifdef HAVE_VSNPRINTF
-    vsnprintf(text, STRLEN, format, arglist);
-#else
-    vsprintf(text, format, arglist);
-#endif
-    va_end(arglist);
-#ifdef USE_WIN32
-    win_log(text);
-#else
-    file_putline(outfile, text);
-#endif
+        level, stunnel_process_id(), stunnel_thread_id());
 }
 
 void ioerror(const char *txt) { /* input/output error handler */
