@@ -1,6 +1,6 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2008 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2009 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -400,7 +400,7 @@ int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     return 0;
 }
 
-#endif
+#endif /* USE_WIN32 */
 
 #ifdef USE_OS2
 
@@ -427,9 +427,9 @@ unsigned long stunnel_thread_id(void) {
     return (unsigned long)ppib->pib_ulpid;
 }
 
-int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
+int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     s_log(LOG_DEBUG, "Creating a new thread");
-    if(_beginthread((void(*)(void *))cli, NULL, STACK_SIZE, arg)==-1) {
+    if(_beginthread((void(*)(void *))cli, NULL, arg->opt->stack_size, arg)==-1) {
         ioerror("_beginthread");
         return -1;
     }
@@ -437,6 +437,8 @@ int create_client(int ls, int s, void *arg, void *(*cli)(void *)) {
     return 0;
 }
 
+#endif /* USE_OS2 */
+
 #ifdef _WIN32_WCE
 
 int _beginthread(void (*start_address)(void *),
@@ -456,9 +458,7 @@ void _endthread(void) {
     ExitThread(0);
 }
 
-#endif /* !defined(_WIN32_WCE) */
-
-#endif /* USE_WIN32 */
+#endif /* _WIN32_WCE */
 
 #ifdef DEBUG_STACK_SIZE
 
